@@ -5,7 +5,7 @@ use warnings;
 use utf8;
 use Regexp::Grammars;
 
-our $VERSION = '0.93';
+our $VERSION = '1.007';
 
 sub new {
 	my ($class) = @_;
@@ -18,8 +18,6 @@ sub PT::Lang::X {
     my $code = 'use strict;
 use warnings;
 use utf8;
-
-our $hash0 = {};
     ';
 
     for my $element ( @{ $class->{Class}} ) {
@@ -89,7 +87,98 @@ sub PT::Group::X {
 
     return (       $class->{Comment}
                 || $class->{Function}
-                || $class->{Parent} )->X($className);
+                || $class->{Parent}
+				|| $class->{Packages} )->X($className);
+}
+
+sub PT::Packages::X {
+	my ($class, $className) = @_;
+
+	my @packageList = ($class->{PackageList})->X($className);
+	my $packages = join("\n", @packageList);
+	return $packages;
+}
+
+sub PT::PackageList::X {
+	my ($class, $className) = @_;
+
+	my @packageList;
+	for my $element ( @{$class->{Package}} ) {
+		push @packageList, $element->X($className);
+	}
+
+	return @packageList;
+}
+
+sub PT::Package::X {
+	my ($class, $className) = @_;
+
+	return (	   $class->{PackageWithConstructor}
+            	|| $class->{PackageWithoutConstructor} )->X($className);
+}
+
+sub PT::PackageWithConstructor::X {
+	my ($class, $className) = @_;
+
+	my $object = $class->{Object}->X();
+	my $packageName = $class->{PackageName}->X();
+	my $constructor = $class->{Constructor}->X();
+
+	if(exists $class->{Parameters}) {
+		my @parameters = $class->{Parameters}->X($className);
+		my $parameters = join(",", @parameters);
+
+		my $packageWithConstructor = "use " . $packageName . ";\n"
+									  . "my \$" . $object . " = " . $packageName . "->"
+									  . $constructor . "(" . $parameters . ");\n";
+		return $packageWithConstructor;
+	}
+
+	my $packageWithConstructor = "use " . $packageName . ";\n"
+								 . "my \$" . $object . " = " . $packageName
+								 . "->" . $constructor . "();\n";
+	return $packageWithConstructor;
+}
+
+sub PT::PackageName::X {
+	my ($class, $className) = @_;
+
+	my @packageDir;
+	for my $element ( @{ $class->{PackageDir}} ) {
+		push @packageDir, $element->X($className);
+	}
+
+	my $packageName = join("::", @packageDir);
+	return $packageName;
+}
+
+sub PT::PackageWithoutConstructor::X {
+	my ($class, $className) = @_;
+
+	my $packageName = $class->{PackageName}->X($className);
+	my $packageWithoutConstructor = "use " . $packageName . ";\n";
+	return $packageWithoutConstructor;
+}
+
+sub PT::Constructor::X {
+	my ($class, $className) = @_;
+
+	my $constructor = $class->{''};
+	return $constructor;
+}
+
+sub PT::Object::X {
+	my ($class, $className) = @_;
+
+	my $object = $class->{''};
+	return $object;
+}
+
+sub PT::PackageDir::X {
+	my ($class, $className) = @_;
+
+	my $packageDir = $class->{''};
+	return $packageDir;
 }
 
 sub PT::Parent::X {
@@ -287,10 +376,10 @@ sub PT::UpperRange::X {
 sub PT::IfElse::X {
     my ($class, $className) = @_;
     my $if = $class->{If}->X($className);
-    
+
     my $elsif;
     my $else;
-    
+
     if( exists $class->{ElsIf} ) {
         $elsif = $class->{ElsIf}->X($className);
     }
@@ -315,10 +404,10 @@ sub PT::IfElse::X {
 sub PT::IfElseIf::X {
     my ($class, $className) = @_;
     my $if = $class->{If}->X($className);
-    
+
     my $elsif;
     my $else;
-    
+
     if( exists $class->{ElsIf} ) {
         $elsif = $class->{ElsIf}->X($className);
     }
@@ -403,7 +492,7 @@ sub PT::BoolOperands::X {
 sub PT::ElsIf::X {
     my ($class, $className) = @_;
     my @elsIfChain;
-    
+
     for my $element ( @{$class->{ElsIfChain}} ) {
         push @elsIfChain, $element->X($className);
     }
@@ -441,7 +530,18 @@ sub PT::Statement::X {
                 || $class->{ClassFunctionCall}
                 || $class->{Return}
                 || $class->{Last}
-                || $class->{Next} )->X($className);
+                || $class->{Next}
+				|| $class->{ObjectCall} )->X($className);
+}
+
+sub PT::ObjectCall::X {
+	my ($class, $className) = @_;
+	my $objectCall = "";
+
+	$objectCall .= $class->{ObjectFunctionCall}->X($className);
+	$objectCall .= ";\n";
+
+	return $objectCall;
 }
 
 sub PT::VariableDeclaration::X {
@@ -635,7 +735,8 @@ sub PT::Param::X {
                 || $class->{String}
                 || $class->{VariableName}
                 || $class->{ArrayElement}
-                || $class->{HashElement} )->X($className);
+                || $class->{HashElement}
+				|| $class->{HashRef} )->X($className);
 }
 
 sub PT::Assignment::X {
@@ -653,8 +754,6 @@ sub PT::AccessorAssignment::X {
     my $rhs = $class->{RHS}->X($className);
 
     my $accessorAssignment  = '$class->{"' . $className . '"}->{"'. $variableName .'"} = ' . $rhs .';';
-    $accessorAssignment .= '$hash0->{"' . $className . '"}->{"'. $variableName .'"} = ' . $rhs .';';
-
     return $accessorAssignment;
 }
 
@@ -694,10 +793,10 @@ sub PT::RHS::X {
                 || $class->{ArrayList}
                 || $class->{HashRef}
                 || $class->{ClassAccessor}
-                || $class->{OtherClassAccesor}
                 || $class->{ClassFunctionReturn}
                 || $class->{String}
-                || $class->{STDIN} )->X($className);
+                || $class->{STDIN}
+				|| $class->{ObjectFunctionCall} )->X($className);
 }
 
 sub PT::STDIN::X {
@@ -706,6 +805,24 @@ sub PT::STDIN::X {
     return $stdin;
 }
 
+sub PT::ObjectFunctionCall::X {
+	my ($class, $className) = @_;
+
+	my $object = $class->{Object}->X($className);
+	my $functionName = $class->{FunctionName}->X($className);
+
+	my $objectFunctionCall;
+	if(exists $class->{Parameters}) {
+		my @parameters = $class->{Parameters}->X($className);
+		my $parameters = join(",", @parameters);
+		$objectFunctionCall = "\$" . $object . "->" . $functionName . "(" . $parameters . ")";
+	} else {
+		$objectFunctionCall = "\$" . $object . "->" . $functionName . "()";
+	}
+
+	return $objectFunctionCall;
+}
+
 sub PT::ClassAccessor::X {
     my ($class, $className) = @_;
     my $variableName = $class->{HashKeyStringValue}->X($className);
@@ -714,15 +831,6 @@ sub PT::ClassAccessor::X {
     return $classAccessor;
 }
 
-sub PT::OtherClassAccesor::X {
-    my ($class, $className) = @_;
-    my $variableName = $class->{HashKeyStringValue}->X($className);
-    my $parentClassName = $class->{ClassName}->X($className);
-
-    my $otherClassAccessor = '$hash0->{"'. $parentClassName .'"}->{"'. $variableName .'"}';
-    return $otherClassAccessor;
-}
-
 sub PT::ClassFunctionCall::X {
     my ($class, $className) = @_;
     my $functionName = $class->{FunctionName}->X($className);
@@ -1022,6 +1130,7 @@ sub PT::LogicalOr::X {
 
 my $parser = qr {
     <nocontext:>
+	<debug: off>
 
     <Lang>
     <objrule:  PT::Lang>                       <[Class]>+
@@ -1031,7 +1140,7 @@ my $parser = qr {
     <objrule:  PT::ClassBlock>                 <LBrace> <ClassGroups> <RBrace>
     <objrule:  PT::ClassGroups>                <[Group]>+
 
-    <objrule:  PT::Group>                      <ws: (\s++)*> <Comment> | <Function> | <Parent>
+    <objrule:  PT::Group>                      <ws: (\s++)*> <Comment> | <Function> | <Parent> | <Packages>
 
     <objrule:  PT::Comment>                    [#] <LineComment> @
     <objtoken: PT::LineComment>                .*?
@@ -1039,8 +1148,18 @@ my $parser = qr {
     <objrule:  PT::Parent>                     <TokenParent> <LParen> <ClassNames> <RParen> <SemiColon>
     <objrule:  PT::ClassNames>                 <[ClassName]>+ % <Comma>
 
+	<objrule:  PT::Packages>				   <LParen> <PackageList> <RParen> <SemiColon>
+	<objrule:  PT::PackageList>			       <[Package]>+ % <Comma>
+	<objrule:  PT::Package>					   <PackageWithConstructor> | <PackageWithoutConstructor>
+    <objrule:  PT::PackageWithConstructor>     [!] <Object> <Equal> <PackageName> <Dot> <Constructor> <LParen> <Parameters>? <RParen>
+	<objrule:  PT::PackageName>				   <[PackageDir]>+ % (::)
+    <objrule:  PT::PackageWithoutConstructor>  <PackageName>
+	<objrule:  PT::Constructor>                [a-zA-Z]+?
+    <objrule:  PT::Object>                     [a-zA-Z]+?
+    <objrule:  PT::PackageDir>                 [a-zA-Z]+?
+
     <objrule:  PT::Function>                   <TokenFunction> <FunctionName> <LParen> <FunctionParamList> <RParen> <CodeBlock>
-    <objtoken: PT::FunctionName>               [a-zA-Z]+?
+    <objtoken: PT::FunctionName>               [a-zA-Z_]+?
 
     <objrule:  PT::FunctionParamList>          <EmptyParamList> | <FunctionParams>
     <objtoken: PT::EmptyParamList>             .{0}
@@ -1066,7 +1185,7 @@ my $parser = qr {
     <objrule:  PT::BoolExpression>             <BoolOperands> <BoolOperatorExpression>?
     <objrule:  PT::BoolOperatorExpression>     <BoolOperator> <BoolOperands>
     <objrule:  PT::BoolOperands>               <RealNumber> | <String> | <ScalarVariable> | <ArrayElement> | <HashElement>
-						| <ClassAccessor> | <ClassFunctionReturn> | <FunctionReturn>
+											   | <ClassAccessor> | <ClassFunctionReturn> | <FunctionReturn>
     <objrule:  PT::BoolOperator>               <GreaterThan> | <LessThan> | <Equals> | <GreaterThanEquals> | <LessThanEquals>
                                                | <StringEquals> | <StringNotEquals> | <NotEqulas> | <LogicalAnd> | <LogicalOr>
     <objrule:  PT::ElsIf>                      <[ElsIfChain]>+
@@ -1074,15 +1193,17 @@ my $parser = qr {
     <objrule:  PT::Else>                       <TokenElse> <CodeBlock>
 
     <objrule:  PT::Statement>                  <VariableDeclaration> | <FunctionCall> | <ClassFunctionCall>
-                                               | <Assignment> | <Return> | <Last> | <Next>
+                                               | <ObjectCall> | <Assignment> | <Return> | <Last> | <Next>
     <objrule:  PT::ClassFunctionCall>          <TokenClass> <Dot> <FunctionName> <LParen> <Parameters>? <RParen> <SemiColon>
 
+	<objrule:  PT::ObjectCall>				    <ObjectFunctionCall> <SemiColon>
+
     <objrule:  PT::VariableDeclaration>        <ArrayDeclaration> | <HashDeclaration> | <ScalarDeclaration>
 
     <objrule:  PT::ScalarDeclaration>          <Var> <VariableName> <Equal> <Value> <SemiColon>
 
     <objtoken: PT::Var>                        var
-    <objtoken: PT::VariableName>               [a-zA-Z]+?
+    <objtoken: PT::VariableName>               [a-zA-Z_]+?
     <objrule:  PT::Value>                      <RHS>
     <objtoken: PT::Number>                     [0-9]+
     <objtoken: PT::RealNumber>                 [-]?[0-9]+\.?[0-9]+|[0-9]+
@@ -1095,7 +1216,7 @@ my $parser = qr {
     <objrule:  PT::ArrayList>                  <LBracket> <ListElements> <RBracket>
     <objrule:  PT::ListElements>               .{0} | <[ListElement]>+ % <Comma>
     <objrule:  PT::ListElement>                <RealNumber> | <String> | <ClassFunctionReturn> | <FunctionReturn>
-						| <ArrayElement> | <HashElement> | <ArrayList> | <HashRef> | <VariableName>
+											   | <ArrayElement> | <HashElement> | <ArrayList> | <HashRef> | <VariableName>
 
     <objrule:  PT::HashDeclaration>            <Var> <VariableName> <Equal> <HashRef> <SemiColon>
     <objrule:  PT::HashRef>                    <LBrace> <KeyValuePairs> <RBrace>
@@ -1103,11 +1224,11 @@ my $parser = qr {
     <objrule:  PT::KeyValue>                   <PairKey> <Colon> <PairValue>
     <objrule:  PT::PairKey>                    <Number> | <String>
     <objrule:  PT::PairValue>                  <RealNumber> | <String> | <ClassFunctionReturn> | <FunctionReturn>
-						| <ArrayElement> | <HashElement> | <ArrayList> | <HashRef> | <VariableName>
+											   | <ArrayElement> | <HashElement> | <ArrayList> | <HashRef> | <VariableName>
 
     <objrule:  PT::FunctionCall>               <FunctionName> <LParen> <Parameters>? <RParen> <SemiColon>
     <objrule:  PT::Parameters>                 <[Param]>+ % <Comma>
-    <objrule:  PT::Param>                      <RealNumber> | <String> | <VariableName> | <ArrayElement> | <HashElement>
+    <objrule:  PT::Param>                      <RealNumber> | <String> | <VariableName> | <ArrayElement> | <HashElement> | <HashRef>
 
     <objrule:  PT::Assignment>                 <ScalarAssignment> | <ArrayAssignment> | <HashAssignment> | <AccessorAssignment>
 
@@ -1115,7 +1236,7 @@ my $parser = qr {
     <objtoken: PT::ScalarVariable>             [a-zA-Z]+
     <objrule:  PT::RHS>                        <RealNumber> | <FunctionReturn> | <ArrayElement> | <HashElement>
                                                | <ScalarVariable> | <Calc> | <ArrayList> | <HashRef> | <ClassAccessor>
-                                               | <OtherClassAccesor> | <ClassFunctionReturn> | <String> | <STDIN>
+                                               | <ClassFunctionReturn> | <String> | <STDIN> | <ObjectFunctionCall>
     <objrule:  PT::FunctionReturn>             <FunctionName> <LParen> <Parameters>? <RParen>
     <objrule:  PT::ArrayElement>               <ArrayName> <[ArrayAccess]>+
     <objrule:  PT::ArrayAccess>                <LBracket> <Number> <RBracket>
@@ -1132,7 +1253,6 @@ my $parser = qr {
 
     <objrule:  PT::AccessorAssignment>         <TokenClass> <Dot> <HashKeyStringValue> <Equal> <RHS> <SemiColon>
     <objrule:  PT::ClassAccessor>              <TokenClass> <Dot> <HashKeyStringValue>
-    <objrule:  PT::OtherClassAccesor>          <TokenClass> <Dot> <ClassName> <Dot> <HashKeyStringValue>
     <objrule:  PT::ClassFunctionReturn>        <TokenClass> <Dot> <FunctionName> <LParen> <Parameters>? <RParen>
 
     <objrule:  PT::ArrayAssignment>            <ArrayElement> <Equal> <RHS> <SemiColon>
@@ -1162,6 +1282,8 @@ my $parser = qr {
 
     <objtoken: PT::TokenSTDIN>                 STDIN
 
+	<objrule:  PT::ObjectFunctionCall>		   [!] <Object> <Dot> <FunctionName> <LParen> <Parameters>? <RParen>
+
     <objtoken: PT::LogicalAnd>                 \&\&
     <objtoken: PT::LogicalOr>                  \|\|
     <objtoken: PT::NotEqulas>                  \!=
@@ -1202,6 +1324,7 @@ sub parse {
 }
 
 1;
+
 __END__
 
 =head1 NAME
@@ -1219,21 +1342,37 @@ HL is a programming language.
 
 =head1 EXAMPLE
 
-	class NotePad {
-	    function notePad() {
-	        var text = "NotePad Example";
-	        class.text = text;
-	    }
-	}
-
 	class Main {
-	    parent(NotePad);
+		(JSON,
+		 !mechanize = WWW::Mechanize.new());
 
-	    function main() {
-	        class.notePad();
-	        var text = class.NotePad.text;
-	        print(text, "\n");
-	    }
+		function main() {
+			class.counter = 0;
+
+			var hash = class.returnNumber();
+			var json = encode_json(hash);
+			print(json, "\n");
+
+			var url = "https://metacpan.org/pod/WWW::Mechanize";
+			!mechanize.get(url);
+			var page = !mechanize.text();
+			print(page, "\n");
+		}
+
+		function returnNumber() {
+			var number = {};
+
+			if(class.counter < 10) {
+				var counter = class.counter;
+				counter = counter + 1;
+				class.counter = counter;
+
+				number = {"number" : class.returnNumber()};
+				return number;
+			}
+
+			return class.counter;
+		}
 	}
 
 =head1 AUTHOR
